import requests
from wpypress.utils import extract_pagination_headers


class CategoriesEndpoint:
    def __init__(self, client):
        self.client = client
        self.endpoint = f"{self.client.base_url}/wp-json/wp/v2/categories"

    def list(self, params=None):
        """List all categories (with optional filters) and pagination metadata"""
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.get(self.endpoint, headers=headers, params=params)
        response.raise_for_status()

        pagination = pagination = extract_pagination_headers(response)
        pagination['page'] = int(params.get('page', 1)) if params else 1
        pagination['per_page'] = int(params.get('per_page', 10)) if params else 10

        return response.json(), pagination

    def get(self, category_id):
        """Get a category by ID"""
        url = f"{self.endpoint}/{category_id}"
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()

    def create(self, name, slug=None, description=None, parent=None, **kwargs):
        """Create a new category"""
        data = {'name': name}
        if slug:
            data['slug'] = slug
        if description:
            data['description'] = description
        if parent:
            data['parent'] = parent  # parent category ID
        data.update(kwargs)

        headers = {'Content-Type': 'application/json'}
        if self.client.auth:
            headers.update(self.client.auth.get_headers())

        response = requests.post(self.endpoint, json=data, headers=headers)
        response.raise_for_status()
        return response.json()

    def update(self, category_id, **kwargs):
        """Update an existing category"""
        url = f"{self.endpoint}/{category_id}"
        headers = {'Content-Type': 'application/json'}
        if self.client.auth:
            headers.update(self.client.auth.get_headers())

        response = requests.post(url, json=kwargs, headers=headers)
        response.raise_for_status()
        return response.json()

    def delete(self, category_id):
        """Permanently delete a category"""
        url = f"{self.endpoint}/{category_id}"
        params = {'force': 'true'}
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.delete(url, headers=headers, params=params)
        response.raise_for_status()
        return response.json()



import requests
from wpypress.utils import extract_pagination_headers

class PostsEndpoint:
    def __init__(self, client):
        self.client = client
        self.endpoint = f"{self.client.base_url}/wp-json/wp/v2/posts"

    def list(self, params=None):
        """List posts with optional filters and pagination metadata"""
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.get(self.endpoint, headers=headers, params=params)
        response.raise_for_status()

        pagination = pagination = extract_pagination_headers(response)
        pagination['page'] = int(params.get('page', 1)) if params else 1
        pagination['per_page'] = int(params.get('per_page', 10)) if params else 10

        return response.json(), pagination

    def get(self, post_id=None, slug=None):
        """Get a single post by ID or slug"""
        if slug:
            url = f"{self.endpoint}?slug={slug}"
        else:
            url = f"{self.endpoint}/{post_id}"
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()

    def create(self, title, content, status='publish', categories=None, tags=None, featured_media=None, excerpt=None, **kwargs):
        """Create a new post"""
        data = {
            'title': title,
            'content': content,
            'status': status,
        }
        if excerpt:
            data['excerpt'] = excerpt
        if categories:
            data['categories'] = categories  # List of category IDs
        if tags:
            data['tags'] = tags              # List of tag IDs
        if featured_media:
            data['featured_media'] = featured_media  # Media ID

        data.update(kwargs)
        headers = {
            'Content-Type': 'application/json'
        }
        if self.client.auth:
            headers.update(self.client.auth.get_headers())

        response = requests.post(self.endpoint, json=data, headers=headers)
        response.raise_for_status()
        return response.json()

    def update(self, post_id, **kwargs):
        """Update an existing post"""
        url = f"{self.endpoint}/{post_id}"
        headers = {
            'Content-Type': 'application/json'
        }
        if self.client.auth:
            headers.update(self.client.auth.get_headers())

        response = requests.post(url, json=kwargs, headers=headers)
        response.raise_for_status()
        return response.json()

    def delete(self, post_id, force=False):
        """Delete a post"""
        url = f"{self.endpoint}/{post_id}"
        headers = self.client.auth.get_headers() if self.client.auth else {}
        params = {'force': 'true' if force else 'false'}
        response = requests.delete(url, headers=headers, params=params)
        response.raise_for_status()
        return response.json()



import requests
from wpypress.utils import extract_pagination_headers

class PagesEndpoint:
    def __init__(self, client):
        self.client = client
        self.endpoint = f"{self.client.base_url}/wp-json/wp/v2/pages"

    def list(self, params=None):
        """List pages and pagination metadata"""
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.get(self.endpoint, headers=headers, params=params)
        response.raise_for_status()

        pagination = extract_pagination_headers(response)
        pagination['page'] = int(params.get('page', 1)) if params else 1
        pagination['per_page'] = int(params.get('per_page', 10)) if params else 10

        return response.json(), pagination

    def get(self, page_id=None, slug=None):
        """Get a single page by ID"""
        if slug:
            url = f"{self.endpoint}?slug={slug}"
        else:
            url = f"{self.endpoint}/{page_id}"
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()

    def create(self, title, content, excerpt=None, status='publish', featured_media=None, **kwargs):
        """Create a new page"""
        data = {
            'title': title,
            'content': content,
            'status': status,
        }
        if excerpt:
            data['excerpt'] = excerpt
        if featured_media:
            data['featured_media'] = featured_media

        data.update(kwargs)

        headers = {'Content-Type': 'application/json'}
        if self.client.auth:
            headers.update(self.client.auth.get_headers())

        response = requests.post(self.endpoint, json=data, headers=headers)
        response.raise_for_status()
        return response.json()

    def update(self, page_id, **kwargs):
        """Update an existing page"""
        url = f"{self.endpoint}/{page_id}"
        headers = {'Content-Type': 'application/json'}
        if self.client.auth:
            headers.update(self.client.auth.get_headers())

        response = requests.post(url, json=kwargs, headers=headers)
        response.raise_for_status()
        return response.json()

    def delete(self, page_id, force=False):
        """Delete a page. Param force=False trash the page, otherwise will be permanently deleted"""
        url = f"{self.endpoint}/{page_id}"
        params = {'force': 'true' if force else 'false'}
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.delete(url, headers=headers, params=params)
        response.raise_for_status()
        return response.json()



from wpypress.auth import BasicAuth
from wpypress.endpoints import posts, categories, tags, pages, media, seo

class WPClient:
    def __init__(self, base_url, username=None, password=None):
        self.base_url = base_url.rstrip('/')
        self.auth = BasicAuth(username, password) if username and password else None

        self.posts = posts.PostsEndpoint(self)
        self.categories = categories.CategoriesEndpoint(self)
        self.tags = tags.TagsEndpoint(self)
        self.pages = pages.PagesEndpoint(self)
        self.media = media.MediaEndpoint(self)
        self.seo = seo.SEOEndpoint(self)



import os
import requests
from wpypress.utils import extract_pagination_headers

class MediaEndpoint:
    def __init__(self, client):
        self.client = client
        self.endpoint = f"{self.client.base_url}/wp-json/wp/v2/media"

    def list(self, params=None):
        """List media items and pagination metadata"""
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.get(self.endpoint, headers=headers, params=params)
        response.raise_for_status()

        pagination = extract_pagination_headers(response)
        pagination['page'] = int(params.get('page', 1)) if params else 1
        pagination['per_page'] = int(params.get('per_page', 10)) if params else 10

        return response.json(), pagination

    def get(self, media_id):
        """Get a single media item by ID"""
        url = f"{self.endpoint}/{media_id}"
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()

    def upload(self, file_path, title=None, alt_text=None, description=None):
        """Upload a media file (image, PDF, etc.)"""
        filename = os.path.basename(file_path)
        mime_type = self._guess_mime_type(filename)

        headers = {
            'Content-Disposition': f'attachment; filename="{filename}"',
            'Content-Type': mime_type,
        }
        if self.client.auth:
            headers.update(self.client.auth.get_headers())

        with open(file_path, 'rb') as file_data:
            response = requests.post(self.endpoint, headers=headers, data=file_data)
            response.raise_for_status()
            media = response.json()

        # Optionally update title, alt_text, description
        if title or alt_text or description:
            update_data = {}
            if title:
                update_data['title'] = title
            if alt_text:
                update_data['alt_text'] = alt_text
            if description:
                update_data['description'] = description

            headers = {'Content-Type': 'application/json'}
            if self.client.auth:
                headers.update(self.client.auth.get_headers())

            update_url = f"{self.endpoint}/{media['id']}"
            update_response = requests.post(update_url, json=update_data, headers=headers)
            update_response.raise_for_status()
            return update_response.json()

        return media

    def delete(self, media_id):
        """Permanently delete a media item"""
        url = f"{self.endpoint}/{media_id}"
        params = {'force': 'true'}
        headers = self.client.auth.get_headers() if self.client.auth else {}
        response = requests.delete(url, headers=headers, params=params)
        response.raise_for_status()
        return response.json()

    def _guess_mime_type(self, filename):
        ext = filename.lower().split('.')[-1]
        return {
            'jpg': 'image/jpeg',
            'jpeg': 'image/jpeg',
            'png': 'image/png',
            'gif': 'image/gif',
            'pdf': 'application/pdf',
        }.get(ext, 'application/octet-stream')



import base64

class BasicAuth:
    def __init__(self, username, password):
        self.username = username
        self.password = password

    def get_headers(self):
        token = base64.b64encode(f"{self.username}:{self.password}".encode()).decode()
        return {
            "Authorization": f"Basic {token}"
        }


import requests

class SEOEndpoint:
    def __init__(self, client):
        self.client = client
        self.base_url = self.client.base_url
        self.yoast_check_endpoint = f"{self.base_url}/wp-json/yoast/v1/get_head"

    def update(self, post_id, title=None, description=None, canonical=None, robots=None,
               focus_keyword=None, og_title=None, og_description=None, type='post'):
        """
        Update Yoast SEO metadata for a post or page through standard post meta fields.

        :param post_id: ID of the post or page
        :param type: 'post' or 'page' (default: 'post')
        """
        if type not in ['post', 'page']:
            raise ValueError("type must be either 'post' or 'page'")

        endpoint = f"{self.base_url}/wp-json/wp/v2/{type}s/{post_id}"

        headers = {'Content-Type': 'application/json'}
        if self.client.auth:
            headers.update(self.client.auth.get_headers())

        meta_fields = {}

        if title:
            meta_fields['yoast_wpseo_title'] = title
        if description:
            meta_fields['yoast_wpseo_metadesc'] = description
        if canonical:
            meta_fields['yoast_wpseo_canonical'] = canonical
        if robots:
            meta_fields['yoast_wpseo_robots'] = robots
        if focus_keyword:
            meta_fields['yoast_wpseo_focuskw'] = focus_keyword

        if og_title:
            meta_fields['yoast_wpseo_opengraph-title'] = og_title
        if og_description:
            meta_fields['yoast_wpseo_opengraph-description'] = og_description

        data = {'meta': meta_fields}

        response = requests.post(endpoint, headers=headers, json=data)
        if not response.ok:
            raise Exception(f"Failed to update SEO metadata: {response.status_code} {response.text}")

        return response.json()

    def is_yoast(self):
        """
        Checks if the Yoast SEO plugin is active and exposing its REST API.
        Returns True if available, False otherwise.
        """
        headers = self.client.auth.get_headers() if self.client.auth else {}
        params = {'url': f"{self.client.base_url}/"}

        try:
            response = requests.get(self.yoast_check_endpoint, headers=headers, params=params)
            return response.status_code == 200
        except requests.RequestException:
            return False



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="wpypress",
    version="0.1.3",
    author="Eugenio Petullà",
    author_email="eugenio@codeat.it",
    description="A versatile Python library for seamless interaction with the WordPress REST API, including built-in SEO features and more.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/EugenioPetulla/wpypress",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "requests>=2.25.1",
    ],
)
